from typing import *

import attr

from ..models.index_params_object_type import IndexParamsObjectType
from ..types import UNSET, Unset

T = TypeVar("T", bound="IndexParams")


def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
    if key not in _dict or _dict[key] is None:
        return UNSET
    return _dict[key]


@attr.s(auto_attribs=True)
class IndexParams:
    """
    Attributes:
        object_type (Union[Unset, IndexParamsObjectType]):  Default: IndexParamsObjectType.INDEXPARAMS.
        nlist (Union[Unset, int]): Number of cluster units Default: 128.
        quant (Union[Unset, int]): Number of factors of product quantization
        nbits (Union[Unset, int]): Number of bits in which each low-dimensional vector is stored
        m (Union[Unset, int]): Maximum degree of the node
        ef_construction (Union[Unset, int]): Search scope
        pqm (Union[Unset, int]): Number of factors of product quantization
        ntrees (Union[Unset, int]): The number of methods of space division
    """

    object_type: Union[Unset, IndexParamsObjectType] = IndexParamsObjectType.INDEXPARAMS
    nlist: Union[Unset, int] = 128
    quant: Union[Unset, int] = UNSET
    nbits: Union[Unset, int] = UNSET
    m: Union[Unset, int] = UNSET
    ef_construction: Union[Unset, int] = UNSET
    pqm: Union[Unset, int] = UNSET
    ntrees: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __init__(
        self,
        object_type: Union[Unset, IndexParamsObjectType] = IndexParamsObjectType.INDEXPARAMS,
        nlist: Union[Unset, int] = 128,
        quant: Union[Unset, int] = UNSET,
        nbits: Union[Unset, int] = UNSET,
        m: Union[Unset, int] = UNSET,
        ef_construction: Union[Unset, int] = UNSET,
        pqm: Union[Unset, int] = UNSET,
        ntrees: Union[Unset, int] = UNSET,
    ):
        """ """

        self.object_type = object_type
        self.nlist = nlist
        self.quant = quant
        self.nbits = nbits
        self.m = m
        self.ef_construction = ef_construction
        self.pqm = pqm
        self.ntrees = ntrees

    def to_dict(self) -> Dict[str, Any]:
        object_type: Union[Unset, str] = UNSET
        if not isinstance(self.object_type, Unset):
            object_type = self.object_type.value

        nlist = self.nlist
        quant = self.quant
        nbits = self.nbits
        m = self.m
        ef_construction = self.ef_construction
        pqm = self.pqm
        ntrees = self.ntrees

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if object_type is not UNSET:
            field_dict["object_type"] = object_type
        if nlist is not UNSET:
            field_dict["nlist"] = nlist
        if quant is not UNSET:
            field_dict["quant"] = quant
        if nbits is not UNSET:
            field_dict["nbits"] = nbits
        if m is not UNSET:
            field_dict["M"] = m
        if ef_construction is not UNSET:
            field_dict["efConstruction"] = ef_construction
        if pqm is not UNSET:
            field_dict["PQM"] = pqm
        if ntrees is not UNSET:
            field_dict["ntrees"] = ntrees

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _object_type = unset_if_none(d, "object_type")
        object_type: Union[Unset, IndexParamsObjectType]
        if isinstance(_object_type, Unset):
            object_type = UNSET
        else:
            object_type = IndexParamsObjectType(_object_type)

        nlist = unset_if_none(d, "nlist")

        quant = unset_if_none(d, "quant")

        nbits = unset_if_none(d, "nbits")

        m = unset_if_none(d, "M")

        ef_construction = unset_if_none(d, "efConstruction")

        pqm = unset_if_none(d, "PQM")

        ntrees = unset_if_none(d, "ntrees")

        index_params = cls(
            object_type=object_type,
            nlist=nlist,
            quant=quant,
            nbits=nbits,
            m=m,
            ef_construction=ef_construction,
            pqm=pqm,
            ntrees=ntrees,
        )

        index_params.additional_properties = d
        return index_params

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import *

import attr

from ..models.index import Index
from ..models.index_in_spec_object_type import IndexInSpecObjectType
from ..types import UNSET, Unset

T = TypeVar("T", bound="IndexInSpec")


def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
    if key not in _dict or _dict[key] is None:
        return UNSET
    return _dict[key]


@attr.s(auto_attribs=True)
class IndexInSpec:
    """
    Attributes:
        collection_name (str):
        field_name (str):
        index (Index):
        object_type (Union[Unset, IndexInSpecObjectType]):  Default: IndexInSpecObjectType.INDEXINSPEC.
    """

    collection_name: str
    field_name: str
    index: Index
    object_type: Union[Unset, IndexInSpecObjectType] = IndexInSpecObjectType.INDEXINSPEC
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __init__(
        self,
        collection_name: str,
        field_name: str,
        index: Index,
        object_type: Union[Unset, IndexInSpecObjectType] = IndexInSpecObjectType.INDEXINSPEC,
    ):
        """ """

        self.collection_name = collection_name
        self.field_name = field_name
        self.index = index
        self.object_type = object_type

    def to_dict(self) -> Dict[str, Any]:
        collection_name = self.collection_name
        field_name = self.field_name
        index = self.index.to_dict()

        object_type: Union[Unset, str] = UNSET
        if not isinstance(self.object_type, Unset):
            object_type = self.object_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "collection_name": collection_name,
                "field_name": field_name,
                "index": index,
            }
        )
        if object_type is not UNSET:
            field_dict["object_type"] = object_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        collection_name = d.pop("collection_name")

        field_name = d.pop("field_name")

        index = Index.from_dict(d.pop("index"))

        _object_type = unset_if_none(d, "object_type")
        object_type: Union[Unset, IndexInSpecObjectType]
        if isinstance(_object_type, Unset):
            object_type = UNSET
        else:
            object_type = IndexInSpecObjectType(_object_type)

        index_in_spec = cls(
            collection_name=collection_name,
            field_name=field_name,
            index=index,
            object_type=object_type,
        )

        index_in_spec.additional_properties = d
        return index_in_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import *

import attr

from ..models.collection_in_spec_object_type import CollectionInSpecObjectType
from ..models.collection_schema import CollectionSchema
from ..types import UNSET, Unset

T = TypeVar("T", bound="CollectionInSpec")


def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
    if key not in _dict or _dict[key] is None:
        return UNSET
    return _dict[key]


@attr.s(auto_attribs=True)
class CollectionInSpec:
    """
    Attributes:
        name (str):
        coll_schema (CollectionSchema):
        object_type (Union[Unset, CollectionInSpecObjectType]):  Default: CollectionInSpecObjectType.COLLECTIONINSPEC.
        consistency_level (Union[Unset, str]):  Default: 'Session'.
        approx (Union[Unset, bool]):  Default: True.
    """

    name: str
    coll_schema: CollectionSchema
    object_type: Union[Unset, CollectionInSpecObjectType] = CollectionInSpecObjectType.COLLECTIONINSPEC
    consistency_level: Union[Unset, str] = "Session"
    approx: Union[Unset, bool] = True
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __init__(
        self,
        name: str,
        coll_schema: CollectionSchema,
        object_type: Union[Unset, CollectionInSpecObjectType] = CollectionInSpecObjectType.COLLECTIONINSPEC,
        consistency_level: Union[Unset, str] = "Session",
        approx: Union[Unset, bool] = True,
    ):
        """ """

        self.name = name
        self.coll_schema = coll_schema
        self.object_type = object_type
        self.consistency_level = consistency_level
        self.approx = approx

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        coll_schema = self.coll_schema.to_dict()

        object_type: Union[Unset, str] = UNSET
        if not isinstance(self.object_type, Unset):
            object_type = self.object_type.value

        consistency_level = self.consistency_level
        approx = self.approx

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "coll_schema": coll_schema,
            }
        )
        if object_type is not UNSET:
            field_dict["object_type"] = object_type
        if consistency_level is not UNSET:
            field_dict["consistency_level"] = consistency_level
        if approx is not UNSET:
            field_dict["approx"] = approx

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        coll_schema = CollectionSchema.from_dict(d.pop("coll_schema"))

        _object_type = unset_if_none(d, "object_type")
        object_type: Union[Unset, CollectionInSpecObjectType]
        if isinstance(_object_type, Unset):
            object_type = UNSET
        else:
            object_type = CollectionInSpecObjectType(_object_type)

        consistency_level = unset_if_none(d, "consistency_level")

        approx = unset_if_none(d, "approx")

        collection_in_spec = cls(
            name=name,
            coll_schema=coll_schema,
            object_type=object_type,
            consistency_level=consistency_level,
            approx=approx,
        )

        collection_in_spec.additional_properties = d
        return collection_in_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import *

import attr

from ..models.hit import Hit
from ..models.query_response_object_type import QueryResponseObjectType
from ..types import UNSET, Unset

T = TypeVar("T", bound="QueryResponse")


def unset_if_none(_dict: dict, key: Any) -> Union[Any, Unset]:
    if key not in _dict or _dict[key] is None:
        return UNSET
    return _dict[key]


@attr.s(auto_attribs=True)
class QueryResponse:
    """
    Attributes:
        hits (List[List[Hit]]):
        query_latency (float):
        total_latency (float):
        object_type (Union[Unset, QueryResponseObjectType]):  Default: QueryResponseObjectType.QUERYRESPONSE.
    """

    hits: List[List[Hit]]
    query_latency: float
    total_latency: float
    object_type: Union[Unset, QueryResponseObjectType] = QueryResponseObjectType.QUERYRESPONSE
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __init__(
        self,
        hits: List[List[Hit]],
        query_latency: float,
        total_latency: float,
        object_type: Union[Unset, QueryResponseObjectType] = QueryResponseObjectType.QUERYRESPONSE,
    ):
        """ """

        self.hits = hits
        self.query_latency = query_latency
        self.total_latency = total_latency
        self.object_type = object_type

    def to_dict(self) -> Dict[str, Any]:
        hits = []
        for hits_item_data in self.hits:
            hits_item = []
            for hits_item_item_data in hits_item_data:
                hits_item_item = hits_item_item_data.to_dict()

                hits_item.append(hits_item_item)

            hits.append(hits_item)

        query_latency = self.query_latency
        total_latency = self.total_latency
        object_type: Union[Unset, str] = UNSET
        if not isinstance(self.object_type, Unset):
            object_type = self.object_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "hits": hits,
                "query_latency": query_latency,
                "total_latency": total_latency,
            }
        )
        if object_type is not UNSET:
            field_dict["object_type"] = object_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        hits = []
        _hits = d.pop("hits")
        for hits_item_data in _hits:
            hits_item = []
            _hits_item = hits_item_data
            for hits_item_item_data in _hits_item:
                hits_item_item = Hit.from_dict(hits_item_item_data)

                hits_item.append(hits_item_item)

            hits.append(hits_item)

        query_latency = d.pop("query_latency")

        total_latency = d.pop("total_latency")

        _object_type = unset_if_none(d, "object_type")
        object_type: Union[Unset, QueryResponseObjectType]
        if isinstance(_object_type, Unset):
            object_type = UNSET
        else:
            object_type = QueryResponseObjectType(_object_type)

        query_response = cls(
            hits=hits,
            query_latency=query_latency,
            total_latency=total_latency,
            object_type=object_type,
        )

        query_response.additional_properties = d
        return query_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
